//
//                             DFree
//
//                        Milan Maksimovi
//
//                      maksa@sezampro.co.yu
//
//   Primer uz tekst objavljen u asopisu PC #14, Jun 1996.
//
//   Program u zavisnosti od broja diskova kreira prozor i u njemu
//   procentualno, brojano (u Mb) i grafiki prikazuje zauzetost
//   postojeih HD particija. Moe se lako prepraviti da radi i/ili
//   za flopije, CD, ili mrene diskove (pogledati naznake u sorsu).
//
//   Program i/ili njegove delove moete koristiti, prepravljati i
//   deliti drugima u izvornom ili izvrnom obliku.
//
//   Autor ne odgovara za eventualnu tetu prouzrokovanu pogrenim
//   korienjem ili prepravljanjem ovog programa.
//
//   Beograd, Maj 1996
//

#include <afxwin.h>
#include <math.h>

#define PI  3.14159265359

struct DriveInfo {
    int SizeInMBytes;
    int FreeMBytes;
    int OccMBytes;
    CString DriveLetter;
    float PercFree;
    float PercOcc;
};

class HDInfo {
private:
    DriveInfo* DI;
    int nD;
public:
    HDInfo();
    ~HDInfo();
    void Create();
    int nDisks() { return (nD);};
    DriveInfo& GetDriveInfo(int nDrive) {return DI[nDrive];};
};

HDInfo::HDInfo() {
    Create();
}

void HDInfo::Create() {

    DWORD bitmap = GetLogicalDrives();
    DWORD SPC, BPS, NOFC, TNOC;

    int nTotD = 0;
    int nDx = 0;
    int Dtype;

    if (bitmap)
		do                      // petlja koja broji bitove
            nTotD++;
        while (0 != (bitmap = bitmap&(bitmap-1)));

    int BuffSize = nTotD*4+1;
    char* DIBuff = new char[BuffSize];

    ::GetLogicalDriveStrings(BuffSize, DIBuff);

    CString DS(DIBuff, BuffSize);
    CString DriveString;
    CString* HD = new CString[nTotD];

    for(int i=0;i<nTotD;i++) {
        DriveString = DS.Mid(i*4, 3);
        Dtype = GetDriveType(DriveString.GetBuffer(3));
        if(Dtype==DRIVE_FIXED)  // (Dtype==DRIVE_FIXED)) za HD
                                //         DRIVE_REMOTE  za mrene
                                //         DRIVE_CDROM za CD
            HD[nDx++]=DriveString;
    }

    nD=nDx;

    DI = new DriveInfo[nDx];

    CString DString;

    for(int j=0;j<nD;j++) {

        GetDiskFreeSpace(HD[j], &SPC, &BPS, &NOFC, &TNOC);
        DI[j].SizeInMBytes = TNOC*SPC*BPS/1024/1024;
        DI[j].FreeMBytes = NOFC*SPC*BPS/1024/1024;
        DI[j].OccMBytes = (TNOC-NOFC)*SPC*BPS/1024/1024;
        DI[j].DriveLetter = HD[j];
        DI[j].PercFree = (float)DI[j].FreeMBytes/(float)DI[j].SizeInMBytes*100;
        DI[j].PercOcc = 100 - DI[j].PercFree;
    }

    delete [] HD;
    delete [] DIBuff;
}

HDInfo::~HDInfo() {
    delete [] DI;
}

class CMainWindow : public CFrameWnd {
public:
	CMainWindow();
private:
    HDInfo DiskInfo;
    void OnPaint();
	DECLARE_MESSAGE_MAP()
};

CMainWindow::CMainWindow()
{
	Create(NULL, "DFree");
    SetWindowPos(NULL, 50, 10, 0, 0, NULL);
    ModifyStyle(WS_THICKFRAME|WS_MAXIMIZEBOX, NULL, NULL);
}

BEGIN_MESSAGE_MAP (CMainWindow, CFrameWnd)
    ON_WM_PAINT()
END_MESSAGE_MAP ()

void CMainWindow::OnPaint()
{
    int Depth = DiskInfo.nDisks();

    CRect WndRect, CliRect, PieRect;
    CPoint Center, Start, End;

    GetWindowRect(&WndRect);
    GetClientRect(&CliRect);

    int BarHeight = WndRect.Height() - CliRect.Height();

    int FWidth = 160;
    int FHeight = 120;
    int Spc = 20;
    int VSpc = 10;
    int R = FWidth/2 - Spc;
    int Base;
    float AngPrFree;

    CString IString;
    CString DString;
    DriveInfo DI;

    SetWindowPos(NULL, 50, 50, FWidth, FHeight*Depth+BarHeight, SWP_NOMOVE);

    CPaintDC MainDC(this);

    CPen GreyPen(PS_SOLID, 1, RGB(128, 128, 128));
    CPen RedPen(PS_SOLID, 1, RGB(255, 0, 0));
    CBrush RedBrush(RGB(255, 0, 0));
    CBrush GreenBrush(RGB(0, 255, 0));

    MainDC.SetBkMode(TRANSPARENT);

    for(int i=0;i<Depth;i++){

        DI = DiskInfo.GetDriveInfo(i);

        MainDC.SelectStockObject(ANSI_VAR_FONT);
        MainDC.SetTextColor(RGB(255, 0, 0));

        IString.Format("Occ: %i Mb", DI.OccMBytes);
        MainDC.TextOut(1, i*FHeight, IString);

        IString.Format("%2.1f", DI.PercOcc);
        IString+="%";
        MainDC.TextOut(10, i*FHeight + FHeight/3, IString);

        IString.Format("Free: %i Mb", DI.FreeMBytes);
        MainDC.SetTextColor(RGB(0, 130, 20));
        MainDC.TextOut(1, i*FHeight+15, IString);

        IString.Format("%2.1f", DI.PercFree);
        IString+="%";

        MainDC.TextOut(FWidth-50, i*FHeight + FHeight/3, IString);

        IString.Format(" of: %i Mb", DI.SizeInMBytes);
        MainDC.SetTextColor(RGB(40, 60, 150));
        MainDC.TextOut(FWidth/2, i*FHeight+7, IString);

        MainDC.SelectObject(&GreyPen);

        MainDC.MoveTo(1, (i+1)*FHeight);
        MainDC.LineTo(FWidth, (i+1)*FHeight);

        Base = (i+1)*FHeight-VSpc;

        PieRect.SetRect(Spc, Base-R, Spc+2*R, Base+R);
		Start = CPoint(Spc+2*R, Base);
		End = CPoint(Spc, Base);

        MainDC.SelectObject(&GreenBrush);
		MainDC.Pie(&PieRect, Start, End);

        Start = CPoint::CPoint(Spc+R, Base);

        AngPrFree = 1.8 * DI.PercFree * PI/180;

        Start.Offset(cos(AngPrFree)*R, sin(-AngPrFree)*R);
        End = CPoint::CPoint(Spc, Base);

        if(DI.PercFree!=100) {
            MainDC.SelectObject(&RedBrush);
            MainDC.Pie(&PieRect, Start, End);
        }

        MainDC.SelectStockObject(SYSTEM_FONT);
        IString=DI.DriveLetter.Left(2);
        IString.MakeUpper();
        MainDC.TextOut(Spc+R-5, i*FHeight + FHeight/4, IString);

	}

    GreyPen.DeleteObject();
    RedPen.DeleteObject();
    RedBrush.DeleteObject();
    GreenBrush.DeleteObject();

}


class CTheApp : public CWinApp
{
public:

    BOOL InitInstance();
};

BOOL CTheApp::InitInstance()
{
    m_pMainWnd = new CMainWindow();
    m_pMainWnd->ShowWindow(m_nCmdShow);
    m_pMainWnd->UpdateWindow();

    return TRUE;
}

CTheApp theApp;
